'use strict';

// Constants
const PORT = 9000;
const HOST = '0.0.0.0';
const API_HOST = 'https://api.listenai.com';
const API_PREFIX = '/v1';
const PLATFORM_REQUEST_KEY = 'e60033bc-36bd-4d99-b44a-77143fec29ff';
const PLATFORM_CHAT_COMPLETIONS = `${API_HOST}${API_PREFIX}/chat/completions`;

const express = require('express');
const superagent = require('superagent');
const app = express();
app.use(express.json());

// 对话所需的参数表单
app.get('/chat/params_form', (req, res) => {
  console.log('/chat/params_form success!')
  res.send({
    name: {
      format: 'input',
      title: '名字',
      type: 'string',
      required: true
    },
    topic: {
      format: 'select',
      title: '话题',
      type: 'string',
      required: true,
      enum: [{
        name: '电影',
        value: 'film'
      }, {
        name: '运动',
        value: 'sport'
      }, {
        name: '美食',
        value: 'food'
      }],
      default: 'film'
    }
  });
});

// 对话
app.use('/chat/completions', async (req, res) => {
  try {
    console.log('requestBody => ', req.body)
    
    const inputs = req.body.inputs || {}
    let messages = req.body.messages || [
      {
        "role": "user",
        "content": ""
      }
    ]

    // 保留n轮对话记录
    // 算法：n(用户 + 系统) + 用户
    // 保留3轮
    const n = 3
    messages = truncateArray(messages, n*(1+1)+1)

    // 用户实际请求的query文本
    var content = messages[messages.length - 1].content || ""

    var prompt = `
      我的名字叫 ${inputs.name} , 我们来进行关于 ${inputs.topic} 的对话。
      这是我的对话内容: ${content} 。
      请你基于上述场景继续我们的对话。
    `

    // 替换拼接好的prompt
    messages[messages.length - 1].content = prompt


    // 拼接请求body
    var body = {
      "uid": req.body.uid,
      "max_tokens": 4096,
      "messages": messages,
      "stream": req.body.stream
    }

    // 请求大模型
    var forwardRequest = superagent
      .post(PLATFORM_CHAT_COMPLETIONS)
      .send(body)
      .set('authorization', `Bearer ${PLATFORM_REQUEST_KEY}`);
    
    if (body.stream) {
      res.setHeader('Content-Type', 'text/event-stream');
    }
    forwardRequest.pipe(res)
  } catch (error) {
    console.error(err);
    return res.status(500).send({
      statusCode: 500,
      message: "请求异常"
    })
  }
});

var server = app.listen(PORT, HOST);
console.log(`Running on http://${HOST}:${PORT}`);

server.timeout = 0; // never timeout
server.keepAliveTimeout = 0; // keepalive, never timeout

function truncateArray(arr, size) {
  if (arr.length >= size) {
    return arr.slice(-size);
  } else {
    return arr;
  }
}